from opendr.renderer import TexturedRenderer
from opendr.camera import ProjectPoints
from cv2 import Rodrigues
import numpy as np
from opendr.lighting import SphericalHarmonics
from opendr.geometry import VertNormals


def lookat(cam_center, ref_center, up_vector):
    # returns a camera rotation matrix that results in a camera
    # looking at ref_center from cam_center, with the projection of up_vector
    # in the image sensor being y
    fwd = (ref_center - cam_center)
    fwd /= np.linalg.norm(fwd)
    side = np.cross(fwd, up_vector)
    side /= np.linalg.norm(side)
    up = np.cross(side, fwd)
    up /= np.linalg.norm(up)
    rot = np.vstack((side, up, -fwd))
    return(rot)


def create_random_cam(v, f, vt, ft, txt, seed=None):
    if seed is not None:
        np.random.seed(seed)
    trans = np.random.randn(3)
    ref_center = v.mean(axis=0)
    rot = lookat(trans, ref_center, np.asarray([0., -1., 0.]))

    parms = {'rt': Rodrigues(rot)[0].flatten(),
             't': rot.dot(trans),
             'f': [150, 150],
             'c': [100, 100],
             'k': np.zeros(8)}

    # set the camera according to the previous parameters, and assign vertices and faces to it
    cam = ProjectPoints(**parms)

    vn = VertNormals(v=v, f=f)
    renderer = TexturedRenderer(v=v, f=f,
                                vt=vt, ft=ft,
                                vc=np.ones_like(v),
                                camera=cam,
                                frustum={'near': .1, 'far': 10,
                                         'height': 200, 'width': 200},
                                bgcolor=np.array(np.zeros(3)),
                                overdraw=False,
                                texture_image=txt)
    # random spherical harmonics ilumination
    # Ambient light needs a minimum
    sh_coeffs = (2 * np.random.rand(9) - 1)
    sh_coeffs[0] = .7 + .3 * np.random.rand()  # first coeff is ambient
    sh_coeffs[1] = -.7 * np.random.rand()  # coming from above
    renderer.vc = SphericalHarmonics(vn=vn, light_color=np.asarray([1., 1., 1.]),
                                     components=sh_coeffs)
    return(renderer)


if __name__ == '__main__':
    from cv2 import imread
    v = np.load('data/rigidv1.npy')
    f = np.load('data/rigidf1.npy')
    vt = np.load('data/vt.npy')
    ft = np.load('data/ft.npy')
    # txt_im = (imread('data/checker.jpg').astype('float')/256)[:, :, ::-1]
    txt_im = (imread('data/realtxt.png').astype('float')/256)[:, :, ::-1]
    create_random_cam(v, f, vt, ft, txt_im)
