import numpy as np
from os.path import join
from cv2 import imread
from solution.create_random_cam import create_random_cam
import chumpy as ch
import matplotlib.pyplot as plt


def set_viewer(im, rdr):

    plt.ion()
    fig = plt.figure()

    ax0 = fig.add_subplot(131)
    ax0.set_axis_off()
    ax0.imshow(im[:, :, ::-1])
    ax0.set_title('image to fit')

    ax1 = fig.add_subplot(132)
    ax1.set_axis_off()
    axim1 = ax1.imshow(rdr.r[:, :, ::-1])
    ax1.set_title('model')

    ax2 = fig.add_subplot(133)
    ax2.set_axis_off()
    axim2 = ax2.imshow(plt.cm.jet(plt.np.linalg.norm(rdr.r - im, axis=2)))
    ax2.set_title('error (warmer=higher)')

    fig.show()
    return(fig, axim1, axim2)


if __name__ == '__main__':

    import sys
    # set the exercise to 2a 2b 2c 2d 2e
    exercises = ['2a', '2b', '2c', '2d', '2e']
    if len(sys.argv) < 2 or sys.argv[1] not in exercises:
        print('Error: first argument should be one of 2a, 2b, 2c, 2d or 2e')
        sys.exit(1)
    exercise = sys.argv[1]

    data_path = './data/'
    np.random.seed(100)  # set to make results predictable

    from smpl.serialization import load_model
    from opendr.filters import gaussian_pyramid

    # load a smpl model. CHANGE THE PATH TO THE DOWNLOADED FILE!
    smpl_path = join(data_path, 'basicModel_f_lbs_10_207_0_v1.0.0.pkl')
    sm = load_model(smpl_path)

    TODO = sm[0]  # DONT CHANGE. just so that it can run without having the answers

    # load texture vertices and texture faces
    vt = np.load('data/vt.npy')
    ft = np.load('data/ft.npy')

    # load texture image
    txt_im = (imread('data/realtxt.png').astype('float')/256)[:, :, ::-1]

    # set the pose to random
    sm.pose[:] = .1*np.random.randn(72)
    # generate a renderer with random camera and random light
    rdr = create_random_cam(sm, sm.f, vt, ft, txt_im)
    # render an image
    # we will try to replicate the parameters that generated it
    im = rdr.r

    # add uniform noise to the position of the camera
    # different amplitude depending on the exercise
    off = (.1 if exercise == '2a' else .2)*np.random.randn(3)
    rdr.camera.t = rdr.camera.t.r + off

    # in exercise 2d and 
    if exercise == '2d' or exercise == '2e':
        sm.pose[:] = sm.pose[:].r + .1*np.random.randn(72)

    # just set the matplot figure to visualize the optimization
    fig, axim1, axim2 = set_viewer(im, rdr)

    # function that updates the viewer in every optimization step
    def on_step(_):
        axim1.set_data(rdr.r[:, :, ::-1])
        axim2.set_data(plt.cm.jet(plt.np.linalg.norm(rdr.r - im, axis=2)))
        plt.pause(1e-5)

    raw_input('Press enter to start optimization')

    if exercise == '2a' or exercise == '2b':
        # optimize camera position to minimize difference in pixels
        obj = {'data': TODO}
        x0 = [TODO]
        ch.minimize(obj, x0=x0,
                    options={'maxiter': 100, 'disp': True, 'e_3': 1e-5},
                    callback=on_step)
    elif exercise == '2c':
        # optimize camera position to minimize difference in pyramid of pixels
        pyr_obj = gaussian_pyramid(rdr-im, np.array(im.shape), as_list=False,
                                   n_levels=5, label=None, normalization='SSE')
        ch.minimize({'data': pyr_obj}, x0=[rdr.camera.t],
                    options={'maxiter': 100, 'disp': True, 'e_3': 1e-5},
                    callback=on_step)
    elif exercise == '2d':
        # optimize camera position and pose to minimize difference in pyramid of pixels
        pyr_obj = gaussian_pyramid(rdr-im, np.array(im.shape), as_list=False,
                                   n_levels=5, label=None, normalization='SSE')
        ch.minimize({'data': pyr_obj}, x0=[sm.trans, sm.pose],
                    options={'maxiter': 100, 'disp': True, 'e_3': 1e-5},
                    callback=on_step)

    elif exercise == '2e':
        # optimize camera position and pose to minimize difference in pyramid of pixels
        # regularized with a pose prior
        pyr_obj = gaussian_pyramid(rdr-im, np.array(im.shape), as_list=False,
                                   n_levels=5, label=None, normalization='SSE')
        ch.minimize({'data': pyr_obj, TODO}, x0=[sm.trans, sm.pose],
                    options={'maxiter': 100, 'disp': True, 'e_3': 1e-5},
                    callback=on_step)

    raw_input('press enter')
